package org.rcisoft.business.memInfo.service.impl;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import jakarta.servlet.http.HttpServletRequest;
import org.apache.commons.lang3.StringUtils;
import org.rcisoft.alibaba.faceVerify.bean.FaceVerifyAliComp;
import org.rcisoft.alibaba.faceVerify.bean.FaceVerifyRedisBean;
import org.rcisoft.business.memGoldCoinFlow.dao.MemGoldCoinFlowRepository;
import org.rcisoft.business.memGoldCoinFlow.entity.MemGoldCoinFlow;
import org.rcisoft.business.memInfo.bean.MemberGenerateBean;
import org.rcisoft.business.memInfo.bean.MemberInfoRedisBean;
import org.rcisoft.business.memInfo.entity.*;
import org.rcisoft.business.memLeaveMessage.bean.MemLeaveMessageRedisBean;
import org.rcisoft.business.memLeaveMessage.entity.MemLeaveMessage;
import org.rcisoft.business.sysScheduledTaskLog.entity.SysScheduledTaskLog;
import org.rcisoft.business.sysScheduledTaskLog.service.SysScheduledTaskLogService;
import org.rcisoft.core.constant.CyDelStaCons;
import org.rcisoft.core.constant.CyFlagStaCons;
import org.rcisoft.core.exception.CyServiceException;
import org.rcisoft.core.service.CyRedisService;
import org.rcisoft.core.util.*;
import org.rcisoft.core.model.CyPersistModel;
import org.rcisoft.business.memInfo.dao.MemInfoRepository;
import org.rcisoft.business.memInfo.service.MemInfoService;

import org.rcisoft.sys.constant.CyUserCons;
import org.rcisoft.sys.dictionary.entity.DictData;
import org.rcisoft.sys.dictionary.service.DictionaryService;
import org.rcisoft.sys.rbac.user.dao.SysUserRbacRepository;
import org.rcisoft.sys.rbac.user.dto.SysUserRbacDTO;
import org.rcisoft.sys.rbac.user.entity.SysUserRbac;
import org.rcisoft.sys.rbac.user.enums.UserInfoExceptionEnums;
import org.redisson.api.RLock;
import org.redisson.api.RedissonClient;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;
import org.rcisoft.core.model.CyPageInfo;

import java.io.UnsupportedEncodingException;
import java.math.BigInteger;
import java.net.URLDecoder;
import java.nio.charset.StandardCharsets;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.LocalTime;
import java.time.format.DateTimeFormatter;
import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

import lombok.extern.slf4j.Slf4j;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

/**
 * Created by cy on 2024年12月20日 下午3:35:34.
 */
@Service
@Transactional(readOnly = true,propagation = Propagation.NOT_SUPPORTED)
@Slf4j
public class MemInfoServiceImpl extends ServiceImpl<MemInfoRepository,MemInfo>  implements MemInfoService {

    // 定义允许的字符集
    private static final String CHAR_POOL = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789_";
    private static final Random RANDOM = new Random();
    private static final Set<String> generatedNicknames = new HashSet<>();

    @Autowired
    private SysUserRbacRepository sysUserRbacRepository;
    @Autowired
    private PasswordEncoder passwordEncoder;
    @Autowired
    CyIdGenUtil cyIdGenUtil;
    @Autowired
    MemberGenerateBean memberGenerateBean;
    @Autowired
    private FaceVerifyAliComp faceVerifyAliComp;
    @Autowired
    private CyRedisService cyRedisServiceImpl;
    @Autowired
    private DictionaryService dictionaryService;
    @Autowired
    private CyRedisService cyRedisService;
    @Autowired
    private StringRedisTemplate redisTemplate;
    @Autowired
    private MemInfoRepository memInfoRepository;
    @Autowired
    private MemGoldCoinFlowRepository memGoldCoinFlowRepository;
    @Autowired
    private SysScheduledTaskLogService sysScheduledTaskLogServiceImpl;
    @Autowired
    private RedissonClient redissonClient;

    @Value("${cy.init.password}")
    private String password;


    /**
    * 保存 会员表
    * @param memInfo
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel persist(MemInfo memInfo){
        //手机号、身份证存储加密
        if (StringUtils.isNotBlank(memInfo.getMemPhone()))
            memInfo.setMemPhone(CyAESUtils.encrypt(memInfo.getMemPhone()));
        if (StringUtils.isNotBlank(memInfo.getMemIdcard()))
            memInfo.setMemIdcard(CyAESUtils.encrypt(memInfo.getMemIdcard()));
        if (StringUtils.isNotBlank(memInfo.getMemRealName()))
            memInfo.setMemRealName(CyAESUtils.encrypt(memInfo.getMemRealName()));
        //身份证、手机号验重
        SysUserRbacDTO sysUserRbacDTO = baseMapper.checkPhoneRepeat(null,memInfo.getMemPhone());
        if (sysUserRbacDTO != null) {
            throw new CyServiceException(UserInfoExceptionEnums.PHONE_EXISTS);
        }
        sysUserRbacDTO = baseMapper.checkIdcardRepeat(null,memInfo.getMemIdcard());
        if (sysUserRbacDTO != null) {
            throw new CyServiceException(UserInfoExceptionEnums.ID_NUMBER_EXISTS);
        }
        //添加sys_user
        SysUserRbac userRbac = new SysUserRbac();
        userRbac.setUsername(this.generateRandomName());
        userRbac.setUserType(CyUserCons.USER_TYPE_COMMON);
        userRbac.setPhone(memInfo.getMemPhone());
        userRbac.setSex(memInfo.getMemSex());
        userRbac.setPassword(passwordEncoder.encode(password));
        userRbac.setIdNumber(memInfo.getMemIdcard());
        userRbac.setUserIdentity("10");
        userRbac.setFlag(CyFlagStaCons.NORMAL.getStatus());
        userRbac.setDelFlag(CyDelStaCons.NORMAL.getStatus());
        userRbac.setWxOpenid(memInfo.getWxOpenid());
//        userRbac.setAvatar(memInfo.getAvatar());
        userRbac.setNickName(memInfo.getMemNickName());
        sysUserRbacRepository.insert(userRbac);
        //查询会员角色
        List<Integer> roleIds = baseMapper.selectRoleByKey("member");
        //插入角色
        if (!roleIds.isEmpty()) {
            sysUserRbacRepository.insertUserRole(userRbac.getBusinessId(), roleIds);
        }
        //生成会员号
        String maxCode = baseMapper.selectMaxCode();
        String memCode = generateMemCode(maxCode);
        memInfo.setMemCode(memCode);
        //增加操作
        memInfo.setUserId(userRbac.getBusinessId());
        memInfo.setSort(0);
        memInfo.setIsRecommended(0);
        int line = baseMapper.insert(memInfo);
        log.debug(CyUserUtil.getAuthenUsername()+"新增了ID为"+
            memInfo.getBusinessId()+"的会员表信息");
        return new CyPersistModel(line);
    }

    /**
    * 删除 会员表
    * @param memInfo
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel remove(MemInfo memInfo){
        int line = baseMapper.realDelete(memInfo);
        log.debug(CyUserUtil.getAuthenUsername()+"删除了ID为"+
            memInfo.getBusinessId()+"的会员表信息");
        return new CyPersistModel(line);
    }

    /**
    * 逻辑删除 会员表
    * @param memInfo
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel removeLogical(MemInfo memInfo){
        memInfo.setDeleted();
        int line = baseMapper.deleteById(memInfo);
            log.debug(CyUserUtil.getAuthenUsername()+"逻辑删除了ID为"+
                memInfo.getBusinessId()+"的会员表信息");
            return new CyPersistModel(line);
    }

    /**
    * 修改 会员表
    * @param memInfo
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel merge(MemInfo memInfo){
        //手机号、身份证存储加密
        if (StringUtils.isNotBlank(memInfo.getMemPhone()))
            memInfo.setMemPhone(CyAESUtils.encrypt(memInfo.getMemPhone()));
        if (StringUtils.isNotBlank(memInfo.getMemIdcard()))
            memInfo.setMemIdcard(CyAESUtils.encrypt(memInfo.getMemIdcard()));
        if (StringUtils.isNotBlank(memInfo.getMemRealName()))
            memInfo.setMemRealName(CyAESUtils.encrypt(memInfo.getMemRealName()));
        //身份证、手机号验重
        SysUserRbacDTO sysUserRbacDTO = baseMapper.checkPhoneRepeat(memInfo.getUserId(),memInfo.getMemPhone());
        if (sysUserRbacDTO != null) {
            throw new CyServiceException(UserInfoExceptionEnums.PHONE_EXISTS);
        }
        sysUserRbacDTO = baseMapper.checkIdcardRepeat(memInfo.getUserId(),memInfo.getMemIdcard());
        if (sysUserRbacDTO != null) {
            throw new CyServiceException(UserInfoExceptionEnums.ID_NUMBER_EXISTS);
        }
        //修改sys_user
        SysUserRbac userRbac = new SysUserRbac();
        userRbac.setBusinessId(memInfo.getUserId());
        userRbac.setPhone(memInfo.getMemPhone());
        userRbac.setSex(memInfo.getMemSex());
        userRbac.setFlag(memInfo.getFlag());
        userRbac.setIdNumber(memInfo.getMemIdcard());
//        userRbac.setAvatar(memInfo.getAvatar());
        userRbac.setNickName(memInfo.getMemNickName());
        sysUserRbacRepository.updateById(userRbac);
        int line = baseMapper.updateById(memInfo);
        log.debug(CyUserUtil.getAuthenUsername()+"修改了ID为"+ memInfo.getBusinessId()+"的会员表信息");
        return new CyPersistModel(line);
    }

    /**
    * 根据id查询 会员表
    * @param id
    * @return
    */
    @Override
    public MemInfo findById(int id){
        //获取当前登录人id
        Integer userId = Integer.valueOf(CyUserUtil.getAuthenBusinessId());
        //获取当天时间 例2025/2/20 后获取0220
        LocalDate today = LocalDate.now();
        DateTimeFormatter monthFormatter = DateTimeFormatter.ofPattern("MM");
        DateTimeFormatter dayFormatter = DateTimeFormatter.ofPattern("dd");
        // 格式化并获取月份和日期
        String month = today.format(monthFormatter);
        String day = today.format(dayFormatter);
        String date = month + day;
        //查询redis限制次数
        //获取查看用户详情限制数量
        List<DictData> userLikeConfig = dictionaryService.selectByTypes("user_config");
        DictData dictData = userLikeConfig.stream().filter(item -> item.getDictLabel().equals("look_user_detail_count")).findFirst().orElse(null);
        Integer lookDetailCount = Integer.valueOf(dictData.getDictValue());//每天最大查看用户详情数量
        Map<Object, Object> hmget = cyRedisServiceImpl.hmget(MemberInfoRedisBean.USER_LOOK_DETAIL_COUNT + date + ":" + userId);
        if (hmget.size() >= lookDetailCount) {
            //超出今日查看数量
            //判断是否已查看了目标用户或者查看的是自己，直接放过
            MemInfo memInfo = baseMapper.selectDetailById(id);
            if (hmget.containsKey(String.valueOf(memInfo.getUserId())) || userId.equals(memInfo.getUserId())) {
                //查询我喜欢的 喜欢我的  互相喜欢的数量
                MemInfo memInfo1 =  baseMapper.selectUserFollowCount(memInfo.getUserId());
                memInfo.setFollowMeCount(memInfo1.getFollowMeCount());
                memInfo.setMeFollowCount(memInfo1.getMeFollowCount());
                memInfo.setEachFollowCount(memInfo1.getEachFollowCount());
                return memInfo;
            } else {
                throw new CyServiceException(1001,"今天查看用户详情数量已达上限");
            }
        }
        //没超出今日查看数量
        MemInfo memInfo = baseMapper.selectDetailById(id);
        //排除当前登录人
        if (!userId.equals(memInfo.getUserId())){
            cyRedisServiceImpl.hset(MemberInfoRedisBean.USER_LOOK_DETAIL_COUNT + date + ":"  + userId, String.valueOf(memInfo.getUserId()),1);
        }
        //查询我喜欢的 喜欢我的  互相喜欢的数量
        MemInfo memInfo1 =  baseMapper.selectUserFollowCount(memInfo.getUserId());
        memInfo.setFollowMeCount(memInfo1.getFollowMeCount());
        memInfo.setMeFollowCount(memInfo1.getMeFollowCount());
        memInfo.setEachFollowCount(memInfo1.getEachFollowCount());
        return memInfo;
    }

    /**
    * 分页查询 会员表
    * @param memInfo
    * @return
    */
    @Override
    public IPage<MemInfo> findAllByPagination(CyPageInfo<MemInfo> paginationUtility,
        MemInfo memInfo,String type){
        if (type.equals("app")){
            //小程序
            //去除小程序端首页的登录人显示  不能自己热门推荐自己
            String loginUserId = CyUserUtil.getAuthenBusinessId();
            if (StringUtils.isNotEmpty(loginUserId)){
                memInfo.setLoginUserId(Integer.valueOf(loginUserId));
            }
            //查询会员表集合
            return baseMapper.appQueryMemInfosPaged(paginationUtility,memInfo);
        } else {
            //web端
            if (memInfo.getMinAge()!=null && memInfo.getMaxAge()!=null) {
                memInfo.setBeginBirthday(computeBirthdayBegin(memInfo.getMaxAge()));
                memInfo.setEndBirthday(computeBirthdayEnd(memInfo.getMinAge()));
            }
            //查询会员表集合
            return baseMapper.queryMemInfosPaged(paginationUtility,memInfo);
        }
    }

    /**
     * 分页查询 会员表
     *
     * @param paginationUtility
     * @param memInfo
     * @return
     */
    @Override
    public IPage<MemInfo> findLikeListByPagination(CyPageInfo<MemInfo> paginationUtility, MemInfo memInfo) {
        String loginUserId = CyUserUtil.getAuthenBusinessId();
        if (StringUtils.isNotEmpty(loginUserId)){
            memInfo.setLoginUserId(Integer.valueOf(loginUserId));
        }
        //查询我喜欢的 喜欢我的  互相喜欢的会员列表（应用于小程序端-我的）
        if (StringUtils.isNotEmpty(memInfo.getQueryType())){
            if (memInfo.getQueryType().equals("1")){
                //我喜欢的
                return baseMapper.queryMeFollowPaged(paginationUtility,memInfo);
            } else if(memInfo.getQueryType().equals("2")){
                //喜欢我的
                return baseMapper.queryFollowMePaged(paginationUtility,memInfo);
            } else if(memInfo.getQueryType().equals("3")){
                //互相喜欢的
                return baseMapper.queryEachFollowPaged(paginationUtility,memInfo);
            }
        }
        return paginationUtility;
    }

    /**
    * 查询list 会员表
    * @param memInfo
    * @return
    */
    @Override
    public List<MemInfo> findAll(MemInfo memInfo){
        return baseMapper.queryMemInfos(memInfo);

    }

    /**
    * 导出会员表
    * @return
    */
    @Override
    public List<MemInfo> export(MemInfo memInfo) {
        List<MemInfo> memInfoList = baseMapper.queryMemInfos(memInfo);
        return memInfoList;
    }


    private String generateRandomName() {
        StringBuilder nickname;
        int length = 10;
        do {
            nickname = new StringBuilder(length);
            for (int i = 0; i < length; i++) {
                int index = RANDOM.nextInt(CHAR_POOL.length());
                nickname.append(CHAR_POOL.charAt(index));
            }
        } while (generatedNicknames.contains(nickname.toString()));
        // 存储生成的昵称
        generatedNicknames.add(nickname.toString());
        return "wx_" + nickname.toString();
    }


    /**
     * 校验会员信息
     * @param memInfo
     * @return
     */
    @Override
    public Boolean checkInfo(MemInfo memInfo) {
        //手机号、身份证存储加密
        if (StringUtils.isNotBlank(memInfo.getMemPhone()))
            memInfo.setMemPhone(CyAESUtils.encrypt(memInfo.getMemPhone()));
        if (StringUtils.isNotBlank(memInfo.getMemIdcard()))
            memInfo.setMemIdcard(CyAESUtils.encrypt(memInfo.getMemIdcard()));
        //身份证、手机号验重
        SysUserRbacDTO sysUserRbacDTO = baseMapper.checkPhoneRepeat(memInfo.getUserId(),memInfo.getMemPhone());
        if (sysUserRbacDTO != null) {
            throw new CyServiceException(UserInfoExceptionEnums.PHONE_EXISTS);
        }
        sysUserRbacDTO = baseMapper.checkIdcardRepeat(null,memInfo.getMemIdcard());
        if (sysUserRbacDTO != null) {
            throw new CyServiceException(UserInfoExceptionEnums.ID_NUMBER_EXISTS);
        }
        return true;
    }

    /**
     * 通过年龄计算出生日期开始
     * @param maxAge
     * @return
     */
    private static String computeBirthdayBegin(Integer maxAge) {
        // 获取当前日期
        LocalDate currentDate = LocalDate.now();
        // 计算出生年份：当前年份减去年龄-1
        int birthYear = currentDate.getYear() - maxAge - 1;
        // 计算出生日期：当前日期晚一天
        LocalDate birthDate = currentDate.plusDays(1);
        birthDate = birthDate.withYear(birthYear);
        return birthDate.toString();
    }


    /**
     * 通过年龄计算出生日期结束
     * @param minAge
     * @return
     */
    private static String computeBirthdayEnd(Integer minAge) {
        // 获取当前日期
        LocalDate currentDate = LocalDate.now();
        // 计算出生年份：当前年份减去年龄-1
        int birthYear = currentDate.getYear() - minAge;
        // 计算出生日期：当前日期晚一天
        LocalDate birthDate = currentDate.withYear(birthYear);
        return birthDate.toString();
    }

    /**
     * 生成会员号
     * @param code
     * @return
     */
    public String generateMemCode(String code) {  //将会员号段排序
        List<String> codeList = Arrays.stream(memberGenerateBean.getCodes()).sorted().collect(Collectors.toList());
        if (StringUtils.isNotBlank(code)) {
            int currentNumber = Integer.parseInt(code);
            //获取当前会员号所处的号段
            for (int i = 0; i < codeList.size(); i++) {
                String[] range = codeList.get(i).split("-");
                int minCode = Integer.parseInt(range[0]);
                int maxCode = Integer.parseInt(range[1]);
                if (currentNumber >= minCode && currentNumber <= maxCode) {
                    // 如果当前号码在号段内，返回该号段中的下一个号码
                    int nextNumber = currentNumber + 1;
                    if (nextNumber <= maxCode) {
                        // 获取当前号码的长度
                        int length = code.length();
                        // 格式化为4位数（根据需要调整位数）
                        return String.format("%0" + length + "d", nextNumber);
                    } else {
                        // 当前号码已经是该号段的最大值，返回下一个号段的起始值
                        if ((i + 1) < codeList.size())
                            return codeList.get(i + 1).split("-")[0];
                        throw new CyServiceException("会员号超出设定最大值，请联系管理员");
                    }
                }
            }
            return null;
        } else {
            return codeList.get(0).split("-")[0];
        }
    }



    @Override
    public MemInfo getUserPhone(Integer businessId) {
        MemInfo memInfo = baseMapper.getUserPhone(businessId);
        //存储加密信息解密
        if (StringUtils.isNotBlank(memInfo.getMemPhone()))
            memInfo.setMemPhone(CyAESUtils.decrypt(memInfo.getMemPhone()));
        if (StringUtils.isNotBlank(memInfo.getMemIdcard()))
            memInfo.setMemIdcard(CyAESUtils.decrypt(memInfo.getMemIdcard()));
        if (StringUtils.isNotBlank(memInfo.getMemRealName()))
            memInfo.setMemRealName(CyAESUtils.decrypt(memInfo.getMemRealName()));
        return memInfo;
    }


    @Override
    public IPage<opmArticleDTO> findArticleByPagination(CyPageInfo<MemInfo> paginationUtility,
                                                        opmArticleDTO opmArticleDTO){
        return baseMapper.queryOpmArticlesPaged(paginationUtility,opmArticleDTO);

    }

    @Override
    public peopleNumberDTO getPeopleNumber(peopleNumberDTO peopleNumberDTO ){
        peopleNumberDTO list = new peopleNumberDTO();

        list.setArticleNumber(baseMapper.getArticleNumber(peopleNumberDTO));
        list.setActivityNumber(baseMapper.getActivityNumber(peopleNumberDTO));
        list.setRealNumber(baseMapper.getRealNumber(peopleNumberDTO));
        list.setRegisterNumber(baseMapper.getRegisterNumber(peopleNumberDTO));
        list.setCmsActivityList(baseMapper.getCmsActivityList(peopleNumberDTO));
        list.setVisitCount(baseMapper.getVisitCountList(peopleNumberDTO));

        return list;
    }


    @Override
    public ArticleExamStatus getArticle(ArticleExamStatus articleExamStatus ){

        LocalDate startDate = articleExamStatus.getDate().minusDays(7);
        LocalDate startDateMonth = articleExamStatus.getDate().minusMonths(7);
        LocalDate endDate = articleExamStatus.getDate();

        ArticleExamStatus list = new ArticleExamStatus();

        //审核通过
        list.setArticleExamStatusDayYes(baseMapper.getArticleCountByDateRangeYes(startDate, endDate));
        list.setArticleExamStatusMonthYes(baseMapper.getArticleCountByMonthRangeYes(startDateMonth, endDate));

        //审核未通过
        list.setArticleExamStatusDayNo(baseMapper.getArticleCountByDateRangeNo(startDate, endDate));
        list.setArticleExamStatusMonthNo(baseMapper.getArticleCountByMonthRangeNo(startDateMonth, endDate));

        return list;
    }


    /**
     * 修改 会员表
     * @param memInfo
     * @return
     */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel checkIdCard(MemInfo memInfo){
        MemInfo info = baseMapper.getInfoByUserId(String.valueOf(memInfo.getUserId()));
        if ("1".equals(info.getMemRealAuthen()))
            throw new CyServiceException("用户已实名，无需再次实名认证");
        //手机号、身份证存储加密
//        if (StringUtils.isNotBlank(memInfo.getMemIdcard()))
//            info.setMemIdcard(CyAESUtils.encrypt(memInfo.getMemIdcard()));
//        if (StringUtils.isNotBlank(memInfo.getMemRealName()))
//            info.setMemRealName(CyAESUtils.encrypt(memInfo.getMemRealName()));
        //身份证、手机号验重
        SysUserRbacDTO sysUserRbacDTO = baseMapper.checkIdcardRepeat(memInfo.getUserId(),CyAESUtils.encrypt(memInfo.getMemIdcard()));
        if (sysUserRbacDTO != null) {
            throw new CyServiceException(UserInfoExceptionEnums.ID_NUMBER_EXISTS);
        }
        //验证失败次数
        //获取ip
        HttpServletRequest request = ((ServletRequestAttributes) RequestContextHolder.getRequestAttributes()).getRequest();
        String ip = CyAddressUtils.IpUtils.getIpAddr(request);
        //1. 同一IP，每分钟失败次数
        Object captcha = cyRedisServiceImpl.get(FaceVerifyRedisBean.FACEVERIFY_IP_MINUTES + ip);
        if (captcha != null) {
            int count = (int)captcha;
            if (count >= faceVerifyAliComp.getIpMinCount())
                throw new CyServiceException("实名认证发起频繁，请稍后重试");
        }
        //2. 同一IP，每天失败次数
        captcha = cyRedisServiceImpl.get(FaceVerifyRedisBean.FACEVERIFY_IP_DAY + ip);
        if (captcha != null) {
            int count = (int)captcha;
            if (count >= faceVerifyAliComp.getIpDayCount())
                throw new CyServiceException("实名认证发起频繁，请稍后重试");
        }
        //3. 同一身份证，每分钟失败次数
        captcha = cyRedisServiceImpl.get(FaceVerifyRedisBean.FACEVERIFY_IDCARD_MINUTES + memInfo.getMemIdcard());
        if (captcha != null) {
            int count = (int)captcha;
            if (count >= faceVerifyAliComp.getIdCardMinCount())
                throw new CyServiceException("实名认证发起频繁，请稍后重试");
        }
        //4. 同一身份证，每天失败次数
        captcha = cyRedisServiceImpl.get(FaceVerifyRedisBean.FACEVERIFY_IDCARD_DAY + memInfo.getMemIdcard());
        if (captcha != null) {
            int count = (int)captcha;
            if (count >= faceVerifyAliComp.getIdCardDayCount())
                throw new CyServiceException("实名认证发起频繁，请稍后重试");
        }
        cyRedisServiceImpl.set(FaceVerifyRedisBean.FACEVERIFY_USER+memInfo.getUserId(),memInfo,3600L);
        return new CyPersistModel(1);
    }

    /**
     * 修改 会员表
     * @param memInfo
     * @return
     */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel update(MemInfo memInfo){
        MemInfo info = baseMapper.selectByOpenId(memInfo.getWxOpenid());
        //手机号、身份证存储加密
        if (StringUtils.isNotBlank(memInfo.getMemPhone()))
            memInfo.setMemPhone(CyAESUtils.encrypt(memInfo.getMemPhone()));
        memInfo.setMemIdcard(info.getMemIdcard());
        memInfo.setMemRealName(info.getMemRealName());
        memInfo.setBusinessId(info.getBusinessId());
        //身份证、手机号验重
        SysUserRbacDTO sysUserRbacDTO = baseMapper.checkPhoneRepeat(info.getUserId(),memInfo.getMemPhone());
        if (sysUserRbacDTO != null) {
            throw new CyServiceException(UserInfoExceptionEnums.PHONE_EXISTS);
        }
        //修改sys_user
        SysUserRbac userRbac = new SysUserRbac();
        userRbac.setBusinessId(info.getUserId());
        userRbac.setPhone(memInfo.getMemPhone());
        userRbac.setSex(memInfo.getMemSex());
        userRbac.setNickName(memInfo.getMemNickName());
        sysUserRbacRepository.updateById(userRbac);
        int line = baseMapper.updateById(memInfo);
        log.debug(CyUserUtil.getAuthenUsername()+"修改了ID为"+ memInfo.getBusinessId()+"的会员表信息");
        return new CyPersistModel(line);
    }
    /**
     * 添加关注
     *
     * @param followDTO
     * @return
     */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel addFollow(MemFollowDTO followDTO){
        int line = baseMapper.addFollow(followDTO);
        return new CyPersistModel(line);
    }
    /**
     * 查询是否关注
     *
     * @param followDTO
     * @return
     */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public MemFollowDTO isFollow(MemFollowDTO followDTO){
        //添加实名判断
        String userId = CyUserUtil.getAuthenBusinessId();
        MemInfo memInfo = memInfoRepository.getInfoByUserId(userId);
        if (!"1".equals(memInfo.getMemRealAuthen()))
            throw new CyServiceException("请先进行实名认证");
        return baseMapper.getIsFollow(followDTO);
    }
    /**
     * 取消关注
     *
     * @param followDTO
     * @return
     */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel deleteFollow(MemFollowDTO followDTO){
        Integer line = baseMapper.deleteFollow(followDTO);
        return new CyPersistModel(line);
    }

    /**
     * 用户点赞
     */
    @Override
    public CyPersistModel addLike(MemLikeDTO likeDTO) {
        //获取当前登录用户id
        String userId = CyUserUtil.getAuthenBusinessId();
        //添加实名判断
        MemInfo info = memInfoRepository.getInfoByUserId(userId);
        if (!"1".equals(info.getMemRealAuthen()))
            throw new CyServiceException("请先进行实名认证");
        //获取当前日期 例2025/2/20 获取02和20
        LocalDate today = LocalDate.now();
        DateTimeFormatter monthFormatter = DateTimeFormatter.ofPattern("MM");
        DateTimeFormatter dayFormatter = DateTimeFormatter.ofPattern("dd");
        // 格式化并获取月份和日期
        String month = today.format(monthFormatter);
        String day = today.format(dayFormatter);
        String date = month + day;
        //获取留言每天限制条数
        List<DictData> userLikeConfig = dictionaryService.selectByTypes("user_like_config");
        DictData dictData1 = userLikeConfig.stream().filter(item -> item.getDictLabel().equals("like_limit_count")).findFirst().orElse(null);
        DictData dictData2 = userLikeConfig.stream().filter(item -> item.getDictLabel().equals("pay_like_limit_count")).findFirst().orElse(null);
        DictData dictData3 = userLikeConfig.stream().filter(item -> item.getDictLabel().equals("all_like_limit_count")).findFirst().orElse(null);
        Integer userLikeLimitCount = Integer.valueOf(dictData1.getDictValue());//给每个人最多免费点赞次数限制
        Integer payUserLikeLimitCount = Integer.valueOf(dictData2.getDictValue());//付费的点赞次数限制
        Integer personalAllCount = userLikeLimitCount + payUserLikeLimitCount;//(个人)免费+付费 次数限制
        Integer allCount = Integer.valueOf(dictData3.getDictValue());//(所有人)总共点赞次数限制
        //获取redis中已存储的限制次数
        Object redisUserLikeCount = cyRedisServiceImpl.hget(MemberInfoRedisBean.USER_LIKE + date + ":" + userId, String.valueOf(likeDTO.getTargetId()));
        Object redisUserLikeCountAll = cyRedisServiceImpl.get(MemberInfoRedisBean.USER_LIKE_ALL + date + ":"  + userId);
        //1.进行总数对比
        if (redisUserLikeCountAll != null && (int)redisUserLikeCountAll <= allCount){
            //没超出总次数限制
            //2.进行判断对目标用户的点赞总次数
            if (redisUserLikeCount != null && (int)redisUserLikeCount <= personalAllCount){
                //没超出对目标用户的点赞总次数(免费+付费)
                //3.判断对目标用户的点赞留言次数
                if (redisUserLikeCount != null && (int)redisUserLikeCount >= userLikeLimitCount) {
                    //超出 进行付费点赞
                    //4.判断用户是否选择付费点赞
                    if (likeDTO.getIsPay().equals("1")){
                        //付费点赞
                        //加redis锁
                        boolean isGetLock = false;
                        RLock lock = redissonClient.getLock(MemberInfoRedisBean.USER_LIKE_LOCK + userId);
                        try {
                            isGetLock = lock.tryLock(Long.parseLong(MemberInfoRedisBean.USER_LIKE_LOCK_WAIT_TIME),
                                    Long.parseLong(MemberInfoRedisBean.USER_LIKE_LOCK_LEASE_TIME), TimeUnit.SECONDS);
                            if (isGetLock) {
                                //扣除该用户的金币数量
                                DictData dictData4 = userLikeConfig.stream().filter(item -> item.getDictLabel().equals("pay_count")).findFirst().orElse(null);
                                Integer payCount = Integer.valueOf(dictData4.getDictValue());
                                MemInfo memInfo = memInfoRepository.selectByOpenId(likeDTO.getWxOpenid());
                                if (memInfo.getGoldCoinsCount() - payCount < 0){
                                    throw new CyServiceException(1003,"金币余额不足,请先充值");
                                }
                                memInfo.setGoldCoinsCount(memInfo.getGoldCoinsCount() - payCount);
                                memInfoRepository.updateById(memInfo);
                                //添加消费记录
                                MemGoldCoinFlow memGoldCoinFlow = new MemGoldCoinFlow();
                                memGoldCoinFlow.setType("2");
                                memGoldCoinFlow.setActionType("20");
                                memGoldCoinFlow.setCount(payCount);
                                memGoldCoinFlow.setEndCount(memInfo.getGoldCoinsCount());
                                memGoldCoinFlow.setTargetId(BigInteger.valueOf(likeDTO.getTargetId()));
                                memGoldCoinFlowRepository.insert(memGoldCoinFlow);
                            }
                        } catch (InterruptedException e) {
                            Thread.currentThread().interrupt();
                            isGetLock = false;
                            throw new CyServiceException(1004,"服务器繁忙，请稍后再试");
                        } finally {
                            lock.unlock();
                            log.error("释放成功");
                        }
                    } else {
                        //不进行留言
                        throw new CyServiceException(1001,"今日对该用户点赞次数超出限制");
                    }
                }
                //给被点赞人+1点赞数
                memInfoRepository.addMemLikeCount(likeDTO);
                //没超出免费点赞次数
                this.redisTemplate.opsForHash().increment(MemberInfoRedisBean.USER_LIKE + date + ":"  + userId, String.valueOf(likeDTO.getTargetId()), (double) 1L);
                this.redisTemplate.opsForValue().increment(MemberInfoRedisBean.USER_LIKE_ALL + date + ":"  + userId, (long) 1L);
                return new CyPersistModel(1);
            } else if (redisUserLikeCount == null){
                //今日第一次对该用户进行点赞
                cyRedisServiceImpl.hset(MemberInfoRedisBean.USER_LIKE + date + ":"  + userId, String.valueOf(likeDTO.getTargetId()), 1L);
                this.redisTemplate.opsForValue().increment(MemberInfoRedisBean.USER_LIKE_ALL + date + ":"  + userId, (long) 1L);
                //新增点赞数据
                likeDTO.setUserId(Integer.valueOf(userId));
                likeDTO.setLikeCount(0);
                memInfoRepository.addLike(likeDTO);
                //给被点赞人+1点赞数
                memInfoRepository.addMemLikeCount(likeDTO);
                return new CyPersistModel(1);
            } else {
                throw new CyServiceException(1001,"今日对该用户点赞次数超出限制");
            }
        } else if (redisUserLikeCountAll == null){
            //当天第一次进行点赞
            cyRedisServiceImpl.hset(MemberInfoRedisBean.USER_LIKE + date + ":"  + userId, String.valueOf(likeDTO.getTargetId()), 1L);
            cyRedisServiceImpl.set(MemberInfoRedisBean.USER_LIKE_ALL + date + ":"  + userId,1L, Long.valueOf(48 * 60 * 60));
            cyRedisServiceImpl.expire(MemberInfoRedisBean.USER_LIKE + date + ":"  + userId, 48 * 60 * 60); // 24小时
            //新增点赞数据
            likeDTO.setUserId(Integer.valueOf(userId));
            likeDTO.setLikeCount(0);
            memInfoRepository.addLike(likeDTO);
            //给被点赞人+1点赞数
            memInfoRepository.addMemLikeCount(likeDTO);
            return new CyPersistModel(1);
        } else {
            //超出总次数限制
            throw new CyServiceException(1002,"今日点赞次数超出限制");
        }
    }

    /**
     * 查询排行榜
     * @param memInfo
     * @return
     */
    @Override
    public IPage<MemInfo> findRankByPagination(CyPageInfo<MemInfo> paginationUtility, MemInfo memInfo){
        return baseMapper.queryRankPaged(paginationUtility,memInfo);
    }


    /**
     * 用户点赞 定时任务
     */
    public void userLikeSchedule() {
        log.info("用户点赞数据定时任务执行中...");
        //获取当前日期
        LocalDate today = LocalDate.now();
        DateTimeFormatter monthFormatter = DateTimeFormatter.ofPattern("MM");
        DateTimeFormatter dayFormatter = DateTimeFormatter.ofPattern("dd");
        // 格式化并获取月份和日期  例2025/2/20 date为0220 用来获取redis的key
        String month = today.format(monthFormatter);
        String day = today.format(dayFormatter);
        String date = month + day;

        // 获取今天的最小时间（00:00:00）
        LocalDateTime startOfDay = today.atStartOfDay();
        // 获取今天的最大时间（23:59:59）
        LocalDateTime endOfDay = today.atTime(LocalTime.MAX);
        // 定义日期时间格式
        DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");
        // 最大时间 和 最小时间 用来查询数据库当天的数据 用来落库
        String beginTime = startOfDay.format(formatter);
        String endTime = endOfDay.format(formatter);

        //将redis中存储的点赞数同步到opm_user_like表中
        Set<String> keys = redisTemplate.keys(MemberInfoRedisBean.USER_LIKE + date + ":" + "*");
        //删除1对多的key  并更新数据库
        if (keys != null && !keys.isEmpty()) {
            for (String key : keys) {
                //截取key最后一个:后的值，为userId
                int lastIndex = key.lastIndexOf(':');
                Integer userId = Integer.valueOf(key.substring(lastIndex + 1));
                //获取hash里的所有key  value
                Map<Object, Object> hmget = cyRedisService.hmget(key);
                if (hmget != null && !hmget.isEmpty()) {
                    //遍历map
                    for (Map.Entry<Object, Object> entry : hmget.entrySet()) {
                        //获取目标
                        String targetId = (String) entry.getKey();
                        //获取点赞数
                        Integer likeCount = (Integer) entry.getValue();
                        //将redis中存储的点赞数同步到opm_user_like表中
                        MemLikeDTO memLikeDTO = new MemLikeDTO();
                        memLikeDTO.setUserId(userId);
                        memLikeDTO.setLikeCount(likeCount);
                        memLikeDTO.setTargetId(Integer.valueOf(targetId));
                        memLikeDTO.setBeginTime(beginTime);
                        memLikeDTO.setEndTime(endTime);
                        //对点赞表 进行修改点赞数
                        Integer integer = memInfoRepository.updateLikeCount(memLikeDTO);
                    }
                }
            }
        } else {
            System.out.println("No keys found.");
        }
        SysScheduledTaskLog sysScheduledTaskLog = new SysScheduledTaskLog();
        sysScheduledTaskLog.setTaskName("用户点赞数据定时任务");
        sysScheduledTaskLog.setTaskState("成功");
        sysScheduledTaskLog.setTaskTime("每2小时执行一次");
        sysScheduledTaskLog.setCreateDate(new Date());
        sysScheduledTaskLogServiceImpl.persist(sysScheduledTaskLog);
        log.info("用户点赞数据定时任务结束---");
    }


    /**
     * 分页查询 会员热度表
     * @param memInfo
     * @return
     */
    @Override
    public IPage<MemInfo> findMemHostByPagination(CyPageInfo<MemInfo> paginationUtility,
                                              MemInfo memInfo){
        return baseMapper.queryMemHostPaged(paginationUtility,memInfo);
    }

    /**
     * 索要微信任务
     *
     * @param memUserTaskDTO
     */
    @Override
    public CyPersistModel weChatTask(MemUserTaskDTO memUserTaskDTO) {
        Integer createBy = Integer.valueOf(CyUserUtil.getAuthenBusinessId());
        memUserTaskDTO.setCreateBy(createBy);
        memUserTaskDTO.setCreateDate(new Date());
        //获取数据字典中的消耗金币数
        List<DictData> userLikeConfig = dictionaryService.selectByTypes("user_config");
        DictData dictData1 = userLikeConfig.stream().filter(item -> item.getDictLabel().equals("request_exchange_wechat")).findFirst().orElse(null);
        DictData dictData2 = userLikeConfig.stream().filter(item -> item.getDictLabel().equals("accept_exchange_wechat")).findFirst().orElse(null);
        Integer requestGoldCoinCount = Integer.valueOf(dictData1.getDictValue());//发起请求需消耗的金币数
        Integer acceptGoldCoinCount = Integer.valueOf(dictData2.getDictValue());//接收需消耗的金币数
//        Integer requestGoldCoinCount = Integer.valueOf('1');//发起请求需消耗的金币数
//        Integer acceptGoldCoinCount = Integer.valueOf('1');//接收需消耗的金币数
        if (memUserTaskDTO.getType().equals("1")){
            //发起请求
            //扣除该用户的金币数量
            int line = 0;
            boolean isGetLock = false;
            RLock lock = redissonClient.getLock(MemberInfoRedisBean.USER_WECHAT_TASK_LOCK + memUserTaskDTO.getCreateBy());
            try {
                isGetLock = lock.tryLock(Long.parseLong(MemberInfoRedisBean.USER_WECHAT_TASK_LOCK_WAIT_TIME),
                        Long.parseLong(MemberInfoRedisBean.USER_WECHAT_LOCK_LEASE_TIME), TimeUnit.SECONDS);
                if (isGetLock) {
                    MemInfo memInfo = memInfoRepository.selectByOpenId(memUserTaskDTO.getWxOpenid());
                    if (memInfo.getGoldCoinsCount() - acceptGoldCoinCount < 0){
                        throw new CyServiceException(1003,"金币余额不足,请先充值");
                    }
                    memInfo.setGoldCoinsCount(memInfo.getGoldCoinsCount() - acceptGoldCoinCount);
                    memInfoRepository.updateById(memInfo);
                    //添加索要微信任务
                    memUserTaskDTO.setStatus("0");
                    line = baseMapper.addWeChatTask(memUserTaskDTO);
                }
                lock.unlock();
            } catch (InterruptedException e) {
                Thread.currentThread().interrupt();
                lock.unlock();
                isGetLock = false;
                throw new CyServiceException(1004,"服务器繁忙，请稍后再试");
            }
            return new CyPersistModel(line);
        } else {
            MemUserTaskDTO memUserTaskDTO1 = baseMapper.selectWeChatTask(memUserTaskDTO);
            memUserTaskDTO.setBusinessId(memUserTaskDTO1.getBusinessId());
            //接收请求
            //判断是同意还是拒绝
            int line = 0;
            if (memUserTaskDTO.getStatus().equals("1")){
                //同意
                //扣除该用户的金币数量
                boolean isGetLock = false;
                RLock lock = redissonClient.getLock(MemberInfoRedisBean.USER_WECHAT_TASK_LOCK + memUserTaskDTO.getCreateBy());
                try {
                    isGetLock = lock.tryLock(Long.parseLong(MemberInfoRedisBean.USER_WECHAT_TASK_LOCK_WAIT_TIME),
                            Long.parseLong(MemberInfoRedisBean.USER_WECHAT_LOCK_LEASE_TIME), TimeUnit.SECONDS);
                    if (isGetLock) {
                        MemInfo memInfo = memInfoRepository.selectByOpenId(memUserTaskDTO.getWxOpenid());
                        if (memInfo.getGoldCoinsCount() - requestGoldCoinCount < 0){
                            throw new CyServiceException(1003,"金币余额不足,请先充值");
                        }
                        memInfo.setGoldCoinsCount(memInfo.getGoldCoinsCount() - requestGoldCoinCount);
                        memInfoRepository.updateById(memInfo);
                        //修改微信任务
                        line = baseMapper.updateWeChatTask(memUserTaskDTO);
                        lock.unlock();
                    }
                } catch (InterruptedException e) {
                    Thread.currentThread().interrupt();
                    lock.unlock();
                    isGetLock = false;
                    throw new CyServiceException(1004,"服务器繁忙，请稍后再试");
                }
                return new CyPersistModel(line);
            } else {
                //拒绝
                //修改微信任务
                line = baseMapper.updateWeChatTask(memUserTaskDTO);
            }
            return new CyPersistModel(line);
        }
    }


    /***
     * 查询互换微信任务
     * @param memUserTaskDTO
     */
    @Override
    public MemUserTaskDTO queryWeChatTaskRecord(MemUserTaskDTO memUserTaskDTO) {
        memUserTaskDTO.setCreateBy(Integer.valueOf(CyUserUtil.getAuthenBusinessId()));
        MemUserTaskDTO memUserTaskDTO1 = baseMapper.selectWeChatTask(memUserTaskDTO);
        return memUserTaskDTO1;
    }
}
